@@ -1,9 +1,9 @@
-/*  $Id: conffile.h 9780 2015-01-07 21:03:00Z iulius $
+/*  $Id: conffile.h 9782 2015-01-07 21:34:22Z iulius $
 **
 **  Data structures, functions and cetera used for config file parsing.
 */
 
-#include <inn/defines.h>
+#include "portable/macros.h"
 
 BEGIN_DECLS
 
