@@ -1,4 +1,4 @@
-/*  $Id: inndcomm.h 9780 2015-01-07 21:03:00Z iulius $
+/*  $Id: inndcomm.h 10150 2017-06-05 12:25:44Z iulius $
 **
 **  Here be values used for communicating with the server once it is
 **  running.
@@ -7,7 +7,7 @@
 #ifndef INN_INNDCOMM_H
 #define INN_INNDCOMM_H 1
 
-#include <inn/defines.h>
+#include "inn/portable-macros.h"
 
 BEGIN_DECLS
 
@@ -25,6 +25,8 @@ typedef char ICC_PROTOCOLTYPE ;
 #define SC_SEP		'\001'
 #define SC_MAXFIELDS	6
 
+/* When modifying this list, the innreport_inn.pm file should be updated
+ * at the same time. */
 #define SC_ADDHIST	'a'
 #define SC_ALLOW	'D'
 #define SC_BEGIN	'b'
