@@ -1,4 +1,4 @@
-/*  $Id: host.h 7778 2008-04-17 21:27:22Z iulius $
+/*  $Id: host.h 10283 2018-05-14 12:43:05Z iulius $
 **
 **  The public interface to the Host class.
 **
@@ -40,7 +40,7 @@ extern void mainLogStatus (FILE *fp) ;
  * ARTTIMEOUT is the max amount of time we'll wait for a new article
  *    from INN before considering the connection unused and we'll close
  *    down.
- * RESPTIMEOUT is the max amount of time we'll wait for any reponse
+ * RESPTIMEOUT is the max amount of time we'll wait for any response
  *    from a remote. Past this we'll close down the network connection.
  * INITIALCXNS is the number of Connections to create at Host creation time.
  * MAXCXNS is the maximum number of parallel connections to the
@@ -85,9 +85,9 @@ void hostFlush (Host host) ;
 void hostSendArticle (Host host, Article article) ;
 
 /* return an IP address for the host */
-struct sockaddr *hostIpAddr (Host host, int family) ;
+struct sockaddr *hostIpAddr (Host host) ;
 
-/* Delete all IPv4 addresses from the address list */
+/* Delete all IPv4 addresses from the address list. */
 void hostDeleteIpv4Addr (Host host);
 
 /* mark the current IP address as failed and rotate to the next one */
@@ -153,12 +153,9 @@ bool hostGimmeArticle (Host host, Connection cxn) ;
 /* get the name that INN uses for this host */
 const char *hostPeerName (Host host) ;
 
-/* get the bindaddress */
-const struct sockaddr_in *hostBindAddr(Host host) ;
-#ifdef HAVE_INET6
-const struct sockaddr_in6 *hostBindAddr6(Host host) ;
-int hostAddrFamily (Host host);
-#endif
+/* Get the bindaddress. */
+const char *hostBindAddr (Host host) ;
+const char *hostBindAddr6 (Host host) ;
 
 /* get the username and password for authentication */
 const char *hostUsername (Host host) ;
